/***************************************************************************
 ** This file is part of the generic algorithm library Wiselib.           **
 ** Copyright (C) 2008,2009 by the Wisebed (www.wisebed.eu) project.      **
 **                                                                       **
 ** The Wiselib is free software: you can redistribute it and/or modify   **
 ** it under the terms of the GNU Lesser General Public License as        **
 ** published by the Free Software Foundation, either version 3 of the    **
 ** License, or (at your option) any later version.                       **
 **                                                                       **
 ** The Wiselib is distributed in the hope that it will be useful,        **
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of        **
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
 ** GNU Lesser General Public License for more details.                   **
 **                                                                       **
 ** You should have received a copy of the GNU Lesser General Public      **
 ** License along with the Wiselib.                                       **
 ** If not, see <http://www.gnu.org/licenses/>.                           **
 ***************************************************************************/
#ifndef COLE_VISHKIN_MESSAGE_H
#define COLE_VISHKIN_MESSAGE_H

#include "util/serialization/simple_types.h"

template<typename OsModel_P, typename Radio_P = typename OsModel_P::Radio>
class ColeVishkinMessage {
public:
   typedef OsModel_P OsModel;
   typedef Radio_P Radio;
   typedef typename Radio::block_data_t block_data_t;
   typedef typename Radio::size_t size_t;
   typedef typename Radio::node_id_t node_id_t;
   
   enum {
      CMD_COLOR
   };
        
   ColeVishkinMessage() {
   }
   // --------------------------------------------------------------------
   uint8_t command_type() {
      return wiselib::read<OsModel, block_data_t, uint8_t> (buffer);
   }
   // --------------------------------------------------------------------
   void set_command_type(uint8_t type) {
      wiselib::write<OsModel, block_data_t, uint8_t> (buffer, type);
   }
   // --------------------------------------------------------------------
   uint16_t color() {
      return wiselib::read<OsModel, block_data_t, uint16_t> (buffer + COLOR_POS);
   }
   // --------------------------------------------------------------------
   void set_color(uint16_t number) {
      wiselib::write<OsModel, block_data_t, uint16_t> (buffer + COLOR_POS, number);
   }
   
   size_t size() {
      return _SIZE;
   }
   
   block_data_t* data() {
      return buffer;
   }

private:
   enum data_in_positions {
      COMMAND_TYPE = 0,
      COLOR_POS = 1,
      _SIZE = COLOR_POS + 2
   };

   // --------------------------------------------------------------------
   block_data_t buffer[Radio_P::MAX_MESSAGE_LENGTH];
};

#endif // COLE_VISHKIN_MESSAGE_H

